export declare class CwdError extends Error {
    path: string;
    code: string;
    syscall: "chdir";
    constructor(path: string, code: string);
    get name(): string;
}
//# sourceMappingURL=cwd-error.d.ts.map