export type Reservation = {
    paths: string[];
    dirs: Set<string>;
};
export type Handler = (clear: () => void) => void;
export declare class PathReservations {
    #private;
    reserve(paths: string[], fn: Handler): boolean;
    check(fn: Handler): boolean;
}
//# sourceMappingURL=path-reservations.d.ts.map